<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 20/04/20
 * Time: 10.45
 */

namespace FiloBlu\Rma\Plugin\Model\Service;
use Psr\Log\LoggerInterface;
use Magento\Rma\Model\Rma;
use FiloBlu\Rma\Plugin\Model\RmaModelRmaProcessor;
use FiloBlu\Rma\Model\RmaEmailManagment;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Rma\Model\ResourceModel\Item\CollectionFactory as ItemCollectionFactory;
use Magento\Rma\Model\ResourceModel\Item;
use Magento\Rma\Model\Item as ItemModel;
use Magento\Rma\Model\Grid;
use \Magento\Rma\Model\ResourceModel\Grid as GridResourceModel;

class RmaManagmentProcessor
{

    /**
     * @var RmaModelRmaProcessor
     */
    private $rmaModelRmaProcessor;
    /**
     * @var Rma
     */
    private $rmaModel;
    /**
     * @var RmaEmailManagment
     */
    private $rmaEmailManagment;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var ItemCollectionFactory
     */
    private $itemCollectionFactory;

    /**
     * @var Item
     */
    private $item;
    /**
     * @var ItemModel
     */
    private $itemModel;
    /**
     * @var Grid
     */
    private $grid;
    /**
     * @var GridResourceModel
     */
    private $gridResourceModel;


    /**
     * RmaManagmentProcessor constructor.
     * @param LoggerInterface $logger
     * @param Rma $rmaModel
     * @param RmaModelRmaProcessor $rmaModelRmaProcessor
     * @param RmaEmailManagment $rmaEmailManagment
     * @param OrderRepositoryInterface $orderRepository
     * @param ItemCollectionFactory $itemCollectionFactory
     * @param Item $item
     * @param ItemModel $itemModel
     * @param Grid $grid
     * @param GridResourceModel $gridResourceModel
     */
    public function __construct(
        LoggerInterface $logger,
        Rma $rmaModel,
        RmaModelRmaProcessor $rmaModelRmaProcessor,
        RmaEmailManagment $rmaEmailManagment,
        OrderRepositoryInterface $orderRepository,
        ItemCollectionFactory $itemCollectionFactory,
        Item $item,
        ItemModel $itemModel,
        Grid $grid,
        GridResourceModel $gridResourceModel){
        $this->logger = $logger;
        $this->rmaModelRmaProcessor = $rmaModelRmaProcessor;
        $this->rmaModel = $rmaModel;
        $this->rmaEmailManagment = $rmaEmailManagment;
        $this->orderRepository = $orderRepository;
        $this->itemCollectionFactory = $itemCollectionFactory;
        $this->item = $item;
        $this->itemModel = $itemModel;
        $this->grid = $grid;
        $this->gridResourceModel = $gridResourceModel;
    }


    /**
     * @param \Magento\Rma\Model\Service\RmaManagement $subject
     * @param $result
     * @return \Magento\Rma\Model\Service\RmaManagement
     */
    public function afterSaveRma(\Magento\Rma\Model\Service\RmaManagement $subject, $result)
    {
        if ($result === false) {
            return $subject;
        }

        try {

            $rma = $this->rmaModel->load($result->getEntityId());
            //save data to rmaItem (no provided from api call)
            $this->loadRmaItemsData($rma);
            //save data to rmaGrid (no provided from api call)
            $this->enrichRmaGrid($rma);
            $this->rmaModelRmaProcessor->afterSaveRma($rma,$result);
            if($result->getStatus() == 'pending'){
                $this->rmaEmailManagment->sendRmaEmail($result->getEntityId(),'pending');
            }
            return $result;
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return $result;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return $result;
        }
    }

    /**
     * @param Rma $rma
     */
    private function loadRmaItemsData(Rma $rma){

        try{

            foreach ($rma->getItems() as $item){
                /** @var \Magento\Rma\Model\Item $item */
                $myItem = $this->itemModel->setEntityId($item->getEntityId());
                $taglia =  $rma->getOrder()->getItems()[$item->getOrderItemId()]->getData('product_options')['attributes_info'][0]['value'];
                $myItem->setData('product_name',$rma->getOrder()->getItems()[$item->getOrderItemId()]->getName());
                $myItem->setData('product_admin_name',$rma->getOrder()->getItems()[$item->getOrderItemId()]->getName()."($taglia)");
                $myItem->setData('product_sku', $rma->getOrder()->getItems()[$item->getOrderItemId()]->getSku());
                $myItem->setData('product_admin_sku',$rma->getOrder()->getItems()[$item->getOrderItemId()]->getSku());
                $myItem->setData('product_options',json_encode($rma->getOrder()->getItems()[$item->getOrderItemId()]->getData('product_options')));
                $this->item->save($myItem);
            }
        }catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
        }

    }

    /**
     * @param Rma $rma
     */
    private function enrichRmaGrid(Rma $rma){
       try{
           $orderDate = $rma->getOrder()->getCreatedAt();
           $customerName = $rma->getOrder()->getCustomerName();
           $rmaGrid = $this->grid->setEntityId($rma->getEntityId());
           $rmaGrid->setData('order_date',$orderDate)
               ->setData('customer_name',$customerName);
           $this->gridResourceModel->save($rmaGrid);
       }catch (\Exception $e) {
           $this->logger->critical($e->getMessage());
       }catch (\Throwable $t){
           $this->logger->critical($t->getMessage());
       }
    }





}